@@ -8,9 +8,9 @@ function HistoryListModel(parent) {
     // Variables
     self.lastUpdate = 0;
     self.historyItems = ko.observableArray([])
-    self.showFailed = ko.observable(false);
+    self.showFailed = ko.observable(false).extend({ persist: 'historyShowFailed' });
     self.isLoading = ko.observable(false).extend({ rateLimit: 100 });
-    self.searchTerm = ko.observable('').extend({ rateLimit: { timeout: 200, method: "notifyWhenChangesStop" } });
+    self.searchTerm = ko.observable('').extend({ rateLimit: { timeout: 400, method: "notifyWhenChangesStop" } });
     self.paginationLimit = ko.observable(10).extend({ persist: 'historyPaginationLimit' });
     self.totalItems = ko.observable(0);
     self.pagination = new paginationModel(self);
@@ -110,20 +110,23 @@ function HistoryListModel(parent) {
                 value: newValue
             })
         }
+        // Update pagination and counters
+        self.parent.refresh(true)
     });
 
     // Retry a job
     self.retryJob = function(form) {
         // Adding a extra retry file happens through this special function
         var data = new FormData();
+        data.append("mode", "retry");
         data.append("nzbfile", $(form.nzbFile)[0].files[0]);
-        data.append("job", $('#modal-retry-job input[name="retry_job_id"]').val());
+        data.append("value", $('#modal-retry-job input[name="retry_job_id"]').val());
         data.append("password", $('#retry_job_password').val());
         data.append("apikey", apiKey);
 
         // Add
         $.ajax({
-            url: "./retry_pp",
+            url: "./api",
             type: "POST",
             cache: false,
             processData: false,
@@ -138,14 +141,15 @@ function HistoryListModel(parent) {
         form.reset()
     }
 
-    // Searching in history (rate-limited in decleration)
+    // Searching in history (rate-limited in declaration)
     self.searchTerm.subscribe(function() {
-        // Make sure we refresh
-        self.lastUpdate = 0
-        self.parent.refresh();
         // Go back to page 1
         if(self.pagination.currentPage() != 1) {
+            // This forces a refresh
             self.pagination.moveToPage(1);
+        } else {
+            // Make sure we refresh
+            self.parent.refresh(true);
         }
     })
 
@@ -156,7 +160,6 @@ function HistoryListModel(parent) {
             // Set the loader so it doesn't flicker and then switch
             self.isLoading(true)
             self.searchTerm('');
-            self.parent.refresh()
         }
         // Was it click and the field is empty? Then we focus on the field
         if(event.type == 'mousedown' && self.searchTerm() == '') {
@@ -333,10 +336,10 @@ function HistoryModel(parent, data) {
         return self.script_line();
     });
 
-    // Extra history column
-    self.extraText = ko.pureComputed(function() {
+    // Extra history columns
+    self.showColumn = function(param) {
         // Picked anything?
-        switch(self.parent.parent.extraHistoryColumn()) {
+        switch(param) {
             case 'speed':
                 // Anything to calculate?
                 if(self.historyStatus.bytes() > 0 && self.historyStatus.download_time() > 0) {
@@ -359,7 +362,7 @@ function HistoryModel(parent, data) {
                 return self.historyStatus.size();
         }
         return;
-    })
+    };
 
     // Format completion time
     self.completedOn = ko.pureComputed(function() {
@@ -421,7 +424,7 @@ function HistoryModel(parent, data) {
     // Delete button
     self.deleteSlot = function(item, event) {
         // Confirm?
-        if(!self.parent.parent.confirmDeleteHistory() || confirm(glitterTranslate.removeDow1)) {
+        if(!self.parent.parent.confirmDeleteHistory() || confirm(glitterTranslate.deleteMsg + ":\n" + item.historyStatus.name() + "\n\n" + glitterTranslate.removeDow1)) {
             // Are we still processing and it can be stopped?
             if(item.processingDownload() == 2) {
                 callAPI({
@@ -438,116 +441,14 @@ function HistoryModel(parent, data) {
                     value: self.nzo_id
                 }).then(function(response) {
                     if(response.status) {
-                        // Fade and remove
-                        $(event.currentTarget).parent().parent().fadeOut(fadeOnDeleteDuration, function() {
-                            // Make sure no flickering (if there are more items left) and then remove
-                            self.parent.isLoading(self.parent.totalItems() > 1)
-                            self.parent.historyItems.remove(self);
-                            self.parent.parent.refresh();
-                        })
+                        // Make sure no flickering (if there are more items left) and then remove
+                        self.parent.isLoading(self.parent.totalItems() > 1)
+                        self.parent.historyItems.remove(self);
+                        self.parent.parent.refresh();
                     }
                 });
             }
 
         }
     };
-
-    // User voting
-    self.setUserVote = function(item, event) {
-        // Send vote
-        callAPI({
-            mode: 'queue',
-            name: 'rating',
-            type: 'vote',
-            setting: $(event.target).val(),
-            value: self.nzo_id
-        }).then(function(response) {
-            // Update all info
-            self.updateAllHistory = true;
-            self.parent.parent.refresh(true)
-        })
-    }
-
-    // User rating
-    self.setUserRating = function(item, event) {
-        // Audio or video
-        var changeWhat = 'audio';
-        if($(event.target).attr('name') == 'ratings-video') {
-            changeWhat = 'video';
-        }
-
-        // Only on user-event, not the auto-fired ones
-        if(!event.originalEvent) return;
-
-        // Send vote
-        callAPI({
-            mode: 'queue',
-            name: 'rating',
-            type: changeWhat,
-            setting: $(event.target).val(),
-            value: self.nzo_id
-        }).then(function(response) {
-            // Update all info
-            self.updateAllHistory = true;
-            self.parent.parent.refresh(true)
-        })
-    }
-
-    // User comment
-    self.setUserReport = function(form) {
-        // What are we reporting?
-        var userReport = $(form).find('input[name="rating_flag"]:checked').val();
-        var userDetail = '';
-
-        // Anything selected?
-        if(!userReport) {
-            alert(glitterTranslate.noSelect)
-            return;
-        }
-
-        // Extra info?
-        if(userReport == 'comment') userDetail = $(form).find('input[name="ratings-report-comment"]').val();
-        if(userReport == 'other') userDetail = $(form).find('input[name="ratings-report-other"]').val();
-
-        // Exception for servers
-        if(userReport == 'expired') {
-            // Which server?
-            userDetail = $(form).find('select[name="ratings-report-expired-server"]').val();
-
-            // All?
-            if(userDetail == "") {
-                // Loop over all servers
-                $.each(parent.parent.servers, function(index, server) {
-                    // Set timeout because simultanious requests don't work (yet)
-                    setTimeout(function() {
-                        submitUserReport(server.name)
-                    }, index * 1500)
-                })
-
-            } else {
-                // Just the one server
-                submitUserReport(userDetail)
-            }
-        } else {
-            submitUserReport(userDetail)
-        }
-
-        // After all, close it
-        form.reset();
-        $(form).parent().parent().dropdown('toggle');
-        alert(glitterTranslate.sendThanks)
-
-        function submitUserReport(theDetail) {
-            // Send note
-            callAPI({
-                mode: 'queue',
-                name: 'rating',
-                type: 'flag',
-                setting: userReport,
-                detail: theDetail,
-                value: self.nzo_id
-            })
-        }
-        return false
-    }
 }
\ No newline at end of file
