@@ -34,7 +34,15 @@ from urllib.parse import urlparse
 import configobj
 
 import sabnzbd
-from sabnzbd.constants import CONFIG_VERSION, NORMAL_PRIORITY, DEFAULT_PRIORITY, CONFIG_BACKUP_FILES, DEF_INI_FILE
+from sabnzbd.constants import (
+    CONFIG_VERSION,
+    NORMAL_PRIORITY,
+    DEFAULT_PRIORITY,
+    CONFIG_BACKUP_FILES,
+    CONFIG_BACKUP_HTTPS,
+    DEF_INI_FILE,
+    DEF_SORTER_RENAME_SIZE,
+)
 from sabnzbd.decorators import synchronized
 from sabnzbd.filesystem import clip_path, real_path, create_real_path, renamer, remove_file, is_writable
 
@@ -55,19 +63,29 @@ RE_PARAMFINDER = re.compile(r"""(?:'.*?')|(?:".*?")|(?:[^'",\s][^,]*)""")
 class Option:
     """Basic option class, basic fields"""
 
-    def __init__(self, section: str, keyword: str, default_val: Any = None, add: bool = True, protect: bool = False):
+    def __init__(
+        self,
+        section: str,
+        keyword: str,
+        default_val: Any = None,
+        add: bool = True,
+        public: bool = True,
+        protect: bool = False,
+    ):
         """Basic option
         `section`     : single section for this option
         `keyword`     : keyword in the section
         `default_val` : value returned when no value has been set
         `callback`    : procedure to call when value is successfully changed
-        `protect`     : Do not allow setting via the API (specifically set_dict)
+        `public`      : if this value should be shown in API calls
+        `protect`     : do not allow setting via the API (specifically set_dict)
         """
         self.__section = section
         self.__keyword: str = keyword
         self.__default_val: Any = default_val
         self.__value: Any = None
         self.__callback: Optional[Callable] = None
+        self.__public: bool = public
         self.__protect = protect
 
         # Add myself to the config dictionary
@@ -84,8 +102,11 @@ class Option:
     def get_string(self) -> str:
         return str(self.get())
 
-    def get_dict(self, safe: bool = False) -> Dict[str, Any]:
-        """Return value a dictionary"""
+    def get_dict(self, for_public_api: bool = False) -> Dict[str, Any]:
+        """Return value as a dictionary.
+        Will not show non-public options if needed for the API"""
+        if not self.__public and for_public_api:
+            return {}
         return {self.__keyword: self.get()}
 
     def set_dict(self, values: Dict[str, Any]):
@@ -130,18 +151,19 @@ class OptionNumber(Option):
         self,
         section: str,
         keyword: str,
-        default_val: Union[int, float] = 0,
-        minval: Optional[int] = None,
-        maxval: Optional[int] = None,
+        default_val: float = 0,
+        minval: Optional[float] = None,
+        maxval: Optional[float] = None,
         validation: Optional[Callable] = None,
         add: bool = True,
+        public: bool = True,
         protect: bool = False,
     ):
-        self.__minval: Optional[int] = minval
-        self.__maxval: Optional[int] = maxval
+        self.__minval: Optional[float] = minval
+        self.__maxval: Optional[float] = maxval
         self.__validation: Optional[Callable] = validation
         self.__int: bool = isinstance(default_val, int)
-        super().__init__(section, keyword, default_val, add=add, protect=protect)
+        super().__init__(section, keyword, default_val, add=add, public=public, protect=protect)
 
     def set(self, value: Any):
         """set new value, limited by range"""
@@ -171,8 +193,16 @@ class OptionNumber(Option):
 class OptionBool(Option):
     """Boolean option class, always returns 0 or 1."""
 
-    def __init__(self, section: str, keyword: str, default_val: bool = False, add: bool = True, protect: bool = False):
-        super().__init__(section, keyword, int(default_val), add=add, protect=protect)
+    def __init__(
+        self,
+        section: str,
+        keyword: str,
+        default_val: bool = False,
+        add: bool = True,
+        public: bool = True,
+        protect: bool = False,
+    ):
+        super().__init__(section, keyword, int(default_val), add=add, public=public, protect=protect)
 
     def set(self, value: Any):
         # Store the value as integer, easier to parse when reading the config.
@@ -196,13 +226,15 @@ class OptionDir(Option):
         validation: Optional[Callable] = None,
         writable: bool = True,
         add: bool = True,
+        public: bool = True,
+        protect: bool = False,
     ):
         self.__validation: Optional[Callable] = validation
         self.__root: str = ""  # Base directory for relative paths
         self.__apply_permissions: bool = apply_permissions
         self.__create: bool = create
         self.__writable: bool = writable
-        super().__init__(section, keyword, default_val, add=add)
+        super().__init__(section, keyword, default_val, add=add, public=public, protect=protect)
 
     def get(self) -> str:
         """Return value, corrected for platform"""
@@ -279,12 +311,13 @@ class OptionList(Option):
         default_val: Union[str, List, None] = None,
         validation: Optional[Callable] = None,
         add: bool = True,
+        public: bool = True,
         protect: bool = False,
     ):
         self.__validation: Optional[Callable] = validation
         if default_val is None:
             default_val = []
-        super().__init__(section, keyword, default_val, add=add, protect=protect)
+        super().__init__(section, keyword, default_val, add=add, public=public, protect=protect)
 
     def set(self, value: Union[str, List]) -> Optional[str]:
         """Set the list given a comma-separated string or a list"""
@@ -325,11 +358,12 @@ class OptionStr(Option):
         validation: Optional[Callable] = None,
         add: bool = True,
         strip: bool = True,
+        public: bool = True,
         protect: bool = False,
     ):
         self.__validation: Optional[Callable] = validation
         self.__strip: bool = strip
-        super().__init__(section, keyword, default_val, add=add, protect=protect)
+        super().__init__(section, keyword, default_val, add=add, public=public, protect=protect)
 
     def get_float(self) -> float:
         """Return value converted to a float, allowing KMGT notation"""
@@ -373,9 +407,9 @@ class OptionPassword(Option):
             return "*" * 10
         return ""
 
-    def get_dict(self, safe: bool = False) -> Dict[str, str]:
+    def get_dict(self, for_public_api: bool = False) -> Dict[str, str]:
         """Return value a dictionary"""
-        if safe:
+        if for_public_api:
             return {self.keyword: self.get_stars()}
         else:
             return {self.keyword: self.get()}
@@ -460,7 +494,7 @@ class ConfigServer:
         if not self.displayname():
             self.displayname.set(self.__name)
 
-    def get_dict(self, safe: bool = False) -> Dict[str, Any]:
+    def get_dict(self, for_public_api: bool = False) -> Dict[str, Any]:
         """Return a dictionary with all attributes"""
         output_dict = {}
         output_dict["name"] = self.__name
@@ -469,7 +503,7 @@ class ConfigServer:
         output_dict["port"] = self.port()
         output_dict["timeout"] = self.timeout()
         output_dict["username"] = self.username()
-        if safe:
+        if for_public_api:
             output_dict["password"] = self.password.get_stars()
         else:
             output_dict["password"] = self.password()
@@ -524,7 +558,7 @@ class ConfigCat:
             except KeyError:
                 continue
 
-    def get_dict(self, safe: bool = False) -> Dict[str, Any]:
+    def get_dict(self, for_public_api: bool = False) -> Dict[str, Any]:
         """Return a dictionary with all attributes"""
         output_dict = {}
         output_dict["name"] = self.__name
@@ -541,6 +575,57 @@ class ConfigCat:
         delete_from_database("categories", self.__name)
 
 
+class ConfigSorter:
+    """Class defining a single Sorter"""
+
+    def __init__(self, name, values):
+        self.__name = clean_section_name(name)
+        name = "sorters," + self.__name
+
+        self.order = OptionNumber(name, "order", len(get_sorters()), 0, 100, add=False)
+        self.min_size = OptionStr(name, "min_size", DEF_SORTER_RENAME_SIZE, add=False)
+        self.multipart_label = OptionStr(name, "multipart_label", add=False)
+        self.sort_string = OptionStr(name, "sort_string", add=False)
+        self.sort_cats = OptionList(name, "sort_cats", add=False)
+        self.sort_type = OptionList(name, "sort_type", add=False)
+        self.is_active = OptionBool(name, "is_active", add=False)
+
+        self.set_dict(values)
+        add_to_database("sorters", self.__name, self)
+
+    def set_dict(self, values: Dict[str, Any]):
+        """Set one or more fields, passed as dictionary"""
+        for kw in ("order", "min_size", "multipart_label", "sort_string", "sort_cats", "sort_type", "is_active"):
+            try:
+                value = values[kw]
+                getattr(self, kw).set(value)
+            except KeyError:
+                continue
+
+    def get_dict(self, for_public_api: bool = False) -> Dict[str, Any]:
+        """Return a dictionary with all attributes"""
+        output_dict = {}
+        output_dict["name"] = self.__name
+        output_dict["order"] = self.order()
+        output_dict["min_size"] = self.min_size()
+        output_dict["multipart_label"] = self.multipart_label()
+        output_dict["sort_string"] = self.sort_string()
+        output_dict["sort_cats"] = self.sort_cats()
+        output_dict["sort_type"] = [int(num) for num in self.sort_type()]
+        output_dict["is_active"] = self.is_active()
+        return output_dict
+
+    def delete(self):
+        """Remove from database"""
+        delete_from_database("sorters", self.__name)
+
+    def rename(self, new_name: str):
+        """Update the name and the saved entries"""
+        delete_from_database("sorters", self.__name)
+        self.__name = new_name
+        add_to_database("sorters", self.__name, self)
+
+
 class OptionFilters(Option):
     """Filter list class"""
 
@@ -578,7 +663,7 @@ class OptionFilters(Option):
             return
         self.set(lst)
 
-    def get_dict(self, safe: bool = False) -> Dict[str, str]:
+    def get_dict(self, for_public_api: bool = False) -> Dict[str, str]:
         """Return filter list as a dictionary with keys 'filter[0-9]+'"""
         output_dict = {}
         for n, rss_filter in enumerate(self.get()):
@@ -630,7 +715,7 @@ class ConfigRSS:
                 continue
         self.filters.set_dict(values)
 
-    def get_dict(self, safe: bool = False) -> Dict[str, Any]:
+    def get_dict(self, for_public_api: bool = False) -> Dict[str, Any]:
         """Return a dictionary with all attributes"""
         output_dict = {}
         output_dict["name"] = self.__name
@@ -670,6 +755,7 @@ AllConfigTypes = Union[
     OptionList,
     OptionDir,
     ConfigCat,
+    ConfigSorter,
     ConfigRSS,
     ConfigServer,
 ]
@@ -712,8 +798,11 @@ def get_dconfig(section, keyword, nested=False):
             sect = CFG_DATABASE[section]
         except KeyError:
             return False, {}
+
         if section == "categories":
             data[section] = get_ordered_categories()
+        elif section == "sorters":
+            data[section] = get_ordered_sorters()
         elif section in ("servers", "rss"):
             data[section] = []
             for keyword in sect.keys():
@@ -730,9 +819,9 @@ def get_dconfig(section, keyword, nested=False):
             item = CFG_DATABASE[section][keyword]
         except KeyError:
             return False, {}
-        data = item.get_dict(safe=True)
+        data = item.get_dict(for_public_api=True)
         if not nested:
-            if section in ("servers", "categories", "rss"):
+            if section in ("sorters", "servers", "categories", "rss"):
                 data = {section: [data]}
             else:
                 data = {section: data}
@@ -832,7 +921,7 @@ def _read_config(path, try_backup=False):
 
     # Use CFG data to set values for all static options
     for section in CFG_DATABASE:
-        if section not in ("servers", "categories", "rss"):
+        if section not in ("sorters", "servers", "categories", "rss"):
             for option in CFG_DATABASE[section]:
                 config_option = CFG_DATABASE[section][option]
                 try:
@@ -850,6 +939,9 @@ def _read_config(path, try_backup=False):
     if "servers" in CFG_OBJ:
         for server in CFG_OBJ["servers"]:
             ConfigServer(server, CFG_OBJ["servers"][server])
+    if "sorters" in CFG_OBJ:
+        for sorter in CFG_OBJ["sorters"]:
+            ConfigSorter(sorter, CFG_OBJ["sorters"][sorter])
 
     CFG_MODIFIED = False
     return True, ""
@@ -868,7 +960,7 @@ def save_config(force=False):
         return False
 
     for section in CFG_DATABASE:
-        if section in ("servers", "categories", "rss"):
+        if section in ("sorters", "servers", "categories", "rss"):
             if section not in CFG_OBJ:
                 CFG_OBJ[section] = {}
 
@@ -943,6 +1035,17 @@ def create_config_backup() -> Union[str, bool]:
                     if os.path.isfile(full_path):
                         with open(full_path, "rb") as data:
                             zip_ref.writestr(filename, data.read())
+                for filename, setting in CONFIG_BACKUP_HTTPS.items():
+                    full_path = getattr(sabnzbd.cfg, setting).get_path()
+                    # Only accept HTTPS config files that were successfully loaded by cherrypy on
+                    # startup to protect against last-minute breaking config changes as well as
+                    # inclusion of unrelated files in the backup through manipulated settings.
+                    if full_path and os.path.isfile(full_path) and full_path in sabnzbd.CONFIG_BACKUP_HTTPS_OK:
+                        logging.debug("Adding %s file %s to backup", setting, full_path)
+                        with open(full_path, "rb") as data:
+                            # Add the https cert/key/chain files with a fixed relative filename,
+                            # regardless of where they are actually stored on the filesystem
+                            zip_ref.writestr(filename, data.read())
                 with open(CFG_OBJ.filename, "rb") as data:
                     zip_ref.writestr(DEF_INI_FILE, data.read())
         return clip_path(complete_path)
@@ -965,6 +1068,7 @@ def validate_config_backup(config_backup_data: bytes) -> bool:
 
 def restore_config_backup(config_backup_data: bytes):
     """Restore configuration files from zip file"""
+    global CFG_MODIFIED
     try:
         with io.BytesIO(config_backup_data) as backup_ref:
             with zipfile.ZipFile(backup_ref, "r") as zip_ref:
@@ -977,16 +1081,22 @@ def restore_config_backup(config_backup_data: bytes):
 
                 # Write the rest of the admin files that we want to recover
                 adminpath = sabnzbd.cfg.admin_dir.get_path()
-                for filename in CONFIG_BACKUP_FILES:
+                for filename in CONFIG_BACKUP_FILES + list(CONFIG_BACKUP_HTTPS.keys()):
                     try:
                         zip_ref.getinfo(filename)
                         destination_file = os.path.join(adminpath, filename)
                         logging.debug("Writing backup of %s to %s", filename, destination_file)
                         with open(destination_file, "wb") as destination_ref:
                             destination_ref.write(zip_ref.read(filename))
+                        # For HTTPS config files, point the associated setting to the restored file
+                        if setting := CONFIG_BACKUP_HTTPS.get(filename):
+                            logging.debug("Setting value of %s to restored file %s", setting, filename)
+                            getattr(sabnzbd.cfg, setting).set(filename)
+                            CFG_MODIFIED = True
                     except KeyError:
                         # File not in archive
                         pass
+                save_config()
     except:
         logging.warning(T("Could not restore backup"))
         logging.info("Traceback: ", exc_info=True)
@@ -1000,6 +1110,24 @@ def get_servers() -> Dict[str, ConfigServer]:
         return {}
 
 
+def get_sorters() -> Dict[str, ConfigSorter]:
+    global CFG_DATABASE
+    try:
+        return CFG_DATABASE["sorters"]
+    except KeyError:
+        return {}
+
+
+def get_ordered_sorters() -> List[Dict]:
+    """Return sorters as an ordered list"""
+    database_sorters = get_sorters()
+
+    sorters = [database_sorters[sorter].get_dict() for sorter in database_sorters.keys()]
+    sorters.sort(key=lambda sorter: sorter["order"])
+
+    return sorters
+
+
 def get_categories() -> Dict[str, ConfigCat]:
     """Return link to categories section.
     This section will always contain special category '*'
