@@ -251,11 +251,19 @@ class DirectUnpacker(threading.Thread):
                     extracted = []
 
                     # Are there more files left?
-                    while self.nzo.files and not self.next_sets:
+                    while not self.nzo.removed_from_queue and not self.next_sets:
+                        logging.debug("Direct Unpack for %s waiting for more sets", self.nzo.final_name)
                         with self.next_file_lock:
                             self.next_file_lock.wait()
 
                     # Is there another set to do?
+                    logging.debug(
+                        "Direct Unpack for %s continuing: killed=%s, cur_setname=%s, next_sets=%s",
+                        self.nzo.final_name,
+                        self.killed,
+                        self.cur_setname,
+                        self.next_sets,
+                    )
                     if self.next_sets:
                         # Start new instance
                         nzf = self.next_sets.pop(0)
@@ -276,8 +284,7 @@ class DirectUnpacker(threading.Thread):
                         rarfiles.append(filename)
                 else:
                     # List files we extracted
-                    m = re.search(RAR_EXTRACTED_RE, linebuf_encoded)
-                    if m:
+                    if m := re.search(RAR_EXTRACTED_RE, linebuf_encoded):
                         # In case of flat-unpack, UnRar still prints the whole path (?!)
                         unpacked_file = m.group(2)
                         if cfg.flat_unpack():
@@ -337,6 +344,7 @@ class DirectUnpacker(threading.Thread):
         self.reset_active()
         if self in ACTIVE_UNPACKERS:
             ACTIVE_UNPACKERS.remove(self)
+        logging.debug("Closing DirectUnpack for %s", self.nzo.final_name)
 
         # Set the thread to killed so it never gets restarted by accident
         self.killed = True
@@ -344,10 +352,14 @@ class DirectUnpacker(threading.Thread):
     def have_next_volume(self):
         """Check if next volume of set is available, start
         from the end of the list where latest completed files are
-        Make sure that files are 100% written to disk by checking md5sum
+        Make sure that files are 100% written to disk by checking nzf.assembled
         """
         for nzf_search in reversed(self.nzo.finished_files):
-            if nzf_search.setname == self.cur_setname and nzf_search.vol == (self.cur_volume + 1) and nzf_search.md5sum:
+            if (
+                nzf_search.setname == self.cur_setname
+                and nzf_search.vol == (self.cur_volume + 1)
+                and nzf_search.assembled
+            ):
                 return nzf_search
         return False
 
@@ -435,7 +447,9 @@ class DirectUnpacker(threading.Thread):
         self.cur_volume = 1
 
         # Need to disable buffer to have direct feedback
-        self.active_instance = build_and_run_command(command, windows_unrar_command=True, bufsize=0)
+        self.active_instance = build_and_run_command(
+            command, windows_unrar_command=True, text_mode=False, stdin=subprocess.PIPE
+        )
 
         # Add to runners
         ACTIVE_UNPACKERS.append(self)
