@@ -31,6 +31,7 @@ import time
 import fnmatch
 import stat
 import ctypes
+import random
 from typing import Union, List, Tuple, Any, Dict, Optional, BinaryIO
 
 try:
@@ -67,8 +68,7 @@ def is_listed_ext(ext: str, ext_list: list) -> bool:
     thus return false for extensions such as 'r007' despite the substring match on 'r00').
     """
     for item in ext_list:
-        RE_EXT = sabnzbd.misc.convert_filter(item)
-        if RE_EXT:
+        if RE_EXT := sabnzbd.misc.convert_filter(item):
             try:
                 if len(RE_EXT.match(ext).group()) == len(ext):
                     return True
@@ -121,6 +121,14 @@ def is_writable(path: str) -> bool:
         return True
 
 
+def is_size(filepath: str, size: int) -> bool:
+    """Return True if filepath exists and is specified size"""
+    try:
+        return os.path.getsize(filepath) == size
+    except:
+        return False
+
+
 _DEVICES = (
     "con",
     "prn",
@@ -177,10 +185,13 @@ def has_win_device(filename: str) -> bool:
     return False
 
 
-CH_ILLEGAL = "/"
-CH_LEGAL = "+"
-CH_ILLEGAL_WIN = '\\/<>?*|"\t:'
-CH_LEGAL_WIN = "++{}!@#'+-"
+CH_ILLEGAL = "\0/"
+CH_LEGAL = "_+"
+CH_ILLEGAL_WIN = '\\/<>?*|":'
+CH_LEGAL_WIN = "++{}!@#'-"
+for i in range(1, 32):
+    CH_ILLEGAL_WIN += chr(i)
+    CH_LEGAL_WIN += "_"
 
 
 def sanitize_filename(name: str) -> str:
@@ -614,8 +625,7 @@ def set_chmod(path: str, permissions: int, allow_failures: bool = False):
 def set_permissions(path: str, recursive: bool = True):
     """Give folder tree and its files their proper permissions"""
     if not sabnzbd.WIN32:
-        custom_permissions = sabnzbd.cfg.permissions()
-        if custom_permissions:
+        if custom_permissions := sabnzbd.cfg.permissions():
             # If user set permissions, parse them
             custom_permissions = int(custom_permissions, 8)
 
@@ -725,7 +735,7 @@ def create_all_dirs(path: str, apply_permissions: bool = False) -> Union[str, bo
 
 
 @synchronized(DIR_LOCK)
-def get_unique_dir(path: str, n: int = 0, create_dir: bool = True) -> str:
+def get_unique_dir(path: str, n: int = 0, create_dir: bool = True) -> Union[str, bool]:
     """Determine a unique folder or filename"""
     if not check_mount(path):
         return path
@@ -764,7 +774,8 @@ def listdir_full(input_dir: str, recursive: bool = True) -> List[str]:
     filelist = []
     for root, dirs, files in os.walk(input_dir):
         for file in files:
-            if not sabnzbd.misc.match_str(root, IGNORED_FILES_AND_FOLDERS):
+            # Ignore special folders and resources files created by macOS
+            if not sabnzbd.misc.match_str(root, IGNORED_FILES_AND_FOLDERS) and not file.startswith("._"):
                 filelist.append(os.path.join(root, file))
         if not recursive:
             break
@@ -1212,8 +1223,7 @@ def backup_exists(filename: str) -> bool:
 
 def backup_nzb(nzb_path: str):
     """Backup NZB file, return path to nzb if it was saved"""
-    nzb_backup_dir = sabnzbd.cfg.nzb_backup_dir.get_path()
-    if nzb_backup_dir:
+    if nzb_backup_dir := sabnzbd.cfg.nzb_backup_dir.get_path():
         logging.debug("Saving copy of %s in %s", get_filename(nzb_path), nzb_backup_dir)
         shutil.copy(nzb_path, nzb_backup_dir)
 
@@ -1263,15 +1273,37 @@ def directory_is_writable_with_file(mydir, myfilename):
 
 
 def directory_is_writable(test_dir: str) -> bool:
-    """Checks if dir is writable at all, and (on non-Windows), writable with special chars.
+    """Checks if dir is writable at all, with long filenames, with unicode,
+    and (on non-Windows), writable with special chars.
     Returns True if all OK, otherwise False"""
-    if directory_is_writable_with_file(test_dir, "sab_test.txt"):
-        if not sabnzbd.WIN32 and not directory_is_writable_with_file(test_dir, "sab_test \\ bla :: , bla.txt"):
-            sabnzbd.misc.helpful_warning(
-                T("%s is not writable with special character filenames. This can cause problems."), test_dir
-            )
-            return False
-    else:
+    if not directory_is_writable_with_file(test_dir, "sab_test.txt"):
         sabnzbd.misc.helpful_warning(T("%s is not writable at all. This blocks downloads."), test_dir)
         return False
     return True
+
+
+def check_filesystem_capabilities(test_dir: str) -> bool:
+    """Checks if we can write long and unicode filenames to the given directory.
+    If not on Windows, also check for special chars like \ and :
+    Returns True if all OK, otherwise False"""
+
+    allgood = True  # default return value: all OK
+
+    # long filename; normal filesystems accept 255 byte filenames
+    if not directory_is_writable_with_file(test_dir, "A" * 245 + str(random.randrange(10000, 99999))):
+        sabnzbd.misc.helpful_warning(T("Cannot write a long filename to %s. This can cause problems."), test_dir)
+        allgood = False
+
+    # unicode in filename
+    if not directory_is_writable_with_file(test_dir, "🚀" * 20):
+        sabnzbd.misc.helpful_warning(T("Cannot write a unicode filename to %s. This can cause problems."), test_dir)
+        allgood = False
+
+    # if not on Windows, check special chars like \ and :
+    if not sabnzbd.WIN32 and not directory_is_writable_with_file(test_dir, "sab_test \\ bla :: , bla.txt"):
+        sabnzbd.misc.helpful_warning(
+            T("%s is not writable with special character filenames. This can cause problems."), test_dir
+        )
+        allgood = False
+
+    return allgood
