@@ -25,14 +25,13 @@ import re
 import subprocess
 import logging
 import time
-import zlib
 import io
 import shutil
 import functools
 from typing import Tuple, List, BinaryIO, Optional, Dict, Any, Union
 
 import sabnzbd
-from sabnzbd.encoding import platform_btou, correct_unknown_encoding, ubtou
+from sabnzbd.encoding import correct_unknown_encoding, ubtou
 import sabnzbd.utils.rarfile as rarfile
 from sabnzbd.misc import (
     format_time_string,
@@ -61,11 +60,12 @@ from sabnzbd.filesystem import (
     build_filelists,
     get_filename,
     SEVENMULTI_RE,
+    is_size,
 )
 from sabnzbd.nzbstuff import NzbObject
-from sabnzbd.sorting import SeriesSorter
 import sabnzbd.cfg as cfg
 from sabnzbd.constants import Status, JOB_ADMIN
+from sabnzbd.sorting import Sorter
 
 # Regex globals
 RAR_V3_RE = re.compile(r"\.(?P<ext>part\d*)$", re.I)
@@ -220,18 +220,18 @@ def external_processing(
         "orig_nzb_gz": clip_path(nzb_paths[0]) if nzb_paths else "",
     }
 
+    # Make sure that if we run a Python script it's output is unbuffered, so we can show it to the user
+    if extern_proc.endswith(".py"):
+        extra_env_fields["pythonunbuffered"] = True
+
     try:
         p = build_and_run_command(command, env=create_env(nzo, extra_env_fields))
         sabnzbd.PostProcessor.external_process = p
 
         # Follow the output, so we can abort it
-        proc = p.stdout
-        if p.stdin:
-            p.stdin.close()
-
         lines = []
         while 1:
-            line = platform_btou(proc.readline())
+            line = p.stdout.readline()
             if not line:
                 break
             line = line.strip()
@@ -690,7 +690,6 @@ def rar_extract_core(
 
     elif RAR_PROBLEM:
         # Use only oldest options, specifically no "-or" or "-scf"
-        # Luckily platform_btou has a fallback for non-UTF-8
         command = [
             RAR_COMMAND,
             action,
@@ -723,10 +722,6 @@ def rar_extract_core(
     p = build_and_run_command(command, windows_unrar_command=True)
     sabnzbd.PostProcessor.external_process = p
 
-    proc = p.stdout
-    if p.stdin:
-        p.stdin.close()
-
     nzo.set_action_line(T("Unpacking"), "00/%02d" % numrars)
 
     # Loop over the output from rar!
@@ -738,7 +733,7 @@ def rar_extract_core(
     lines = []
 
     while 1:
-        line = platform_btou(proc.readline())
+        line = p.stdout.readline()
         if not line:
             break
 
@@ -787,7 +782,7 @@ def rar_extract_core(
             fail = 1
 
         elif line.startswith("Cannot create"):
-            line2 = platform_btou(proc.readline())
+            line2 = p.stdout.readline()
             if "must not exceed 260" in line2:
                 msg = "%s: %s" % (T("Unpacking failed, path is too long"), line[13:])
             else:
@@ -843,14 +838,14 @@ def rar_extract_core(
                 extracted.append(real_path(extraction_path, unpacked_file))
 
         if fail:
-            if proc:
-                proc.close()
+            if p.stdout:
+                p.stdout.close()
             p.wait()
             logging.debug("UNRAR output %s", "\n".join(lines))
             return fail, [], []
 
-    if proc:
-        proc.close()
+    if p.stdout:
+        p.stdout.close()
     p.wait()
 
     # Which files did we use to extract this?
@@ -936,10 +931,8 @@ def unzip_core(zipfile, extraction_path, one_folder):
         command.insert(3, "-j")  # Unpack without folders
 
     p = build_and_run_command(command)
-    output = platform_btou(p.stdout.read())
-    ret = p.wait()
-    logging.debug("unzip output: \n%s", output)
-    return ret
+    logging.debug("unzip output: \n%s", p.stdout.read())
+    return p.wait()
 
 
 ##############################################################################
@@ -1054,7 +1047,7 @@ def seven_extract_core(
     command = [SEVENZIP_COMMAND, method, "-y", overwrite, case, password, "-o%s" % extraction_path, seven_path]
     p = build_and_run_command(command)
     sabnzbd.PostProcessor.external_process = p
-    output = platform_btou(p.stdout.read())
+    output = p.stdout.read()
     logging.debug("7za output: %s", output)
 
     # ret contains the 7z/7za exit code: 0 = Normal, 1 = Warning, 2 = Fatal error, etc
@@ -1257,10 +1250,6 @@ def par2cmdline_verify(
     # Run the external command
     p = build_and_run_command(command)
     sabnzbd.PostProcessor.external_process = p
-    proc = p.stdout
-
-    if p.stdin:
-        p.stdin.close()
 
     # Set up our variables
     lines = []
@@ -1282,7 +1271,7 @@ def par2cmdline_verify(
 
     # Loop over the output, whee
     while 1:
-        char = platform_btou(proc.read(1))
+        char = p.stdout.read(1)
         if not char:
             break
 
@@ -1551,16 +1540,13 @@ def multipar_verify(
     # Run MultiPar
     p = build_and_run_command(command)
     sabnzbd.PostProcessor.external_process = p
-    proc = p.stdout
-    if p.stdin:
-        p.stdin.close()
 
     # Set up our variables
     lines = []
     renames = {}
     reconstructed = []
 
-    linebuf = b""
+    linebuf = ""
     finished = False
     readd = False
 
@@ -1577,17 +1563,17 @@ def multipar_verify(
 
     # Loop over the output, whee
     while 1:
-        char = proc.read(1)
+        char = p.stdout.read(1)
         if not char:
             break
 
         # Line not complete yet
-        if char not in (b"\n", b"\r"):
+        if char not in ("\n", "\r"):
             linebuf += char
             continue
 
-        line = ubtou(linebuf).strip()
-        linebuf = b""
+        line = linebuf.strip()
+        linebuf = ""
 
         # Skip empty lines
         if line == "":
@@ -1922,9 +1908,6 @@ def create_env(nzo: Optional[NzbObject] = None, extra_env_fields: Dict[str, Any]
             del env["PYTHONPATH"]
         if "PYTHONHOME" in env:
             del env["PYTHONHOME"]
-    elif not nzo:
-        # No modification
-        return None
 
     return env
 
@@ -1975,7 +1958,7 @@ def rar_sort(a: str, b: str) -> int:
 
 
 def quick_check_set(setname: str, nzo: NzbObject) -> bool:
-    """Check all on-the-fly md5sums of a set"""
+    """Check all on-the-fly crc32s of a set"""
     par2pack = nzo.par2packs.get(setname)
     if par2pack is None:
         return False
@@ -1997,7 +1980,11 @@ def quick_check_set(setname: str, nzo: NzbObject) -> bool:
             # Do a simple filename based check
             if file == nzf.filename:
                 found = True
-                if (nzf.md5sum is not None) and nzf.md5sum == par2info.filehash:
+                if (
+                    nzf.crc32 is not None
+                    and nzf.crc32 == par2info.filehash
+                    and is_size(nzf.filepath, par2info.filesize)
+                ):
                     logging.debug("Quick-check of file %s OK", file)
                     result &= True
                 elif file_to_ignore:
@@ -2010,7 +1997,7 @@ def quick_check_set(setname: str, nzo: NzbObject) -> bool:
                 break
 
             # Now let's do obfuscation check
-            if nzf.md5sum == par2info.filehash:
+            if nzf.crc32 is not None and nzf.crc32 == par2info.filehash and is_size(nzf.filepath, par2info.filesize):
                 try:
                     logging.debug("Quick-check will rename %s to %s", nzf.filename, file)
 
@@ -2155,9 +2142,10 @@ def sfv_check(sfvs: List[str], nzo: NzbObject) -> bool:
     verifytotal = len(nzo.finished_files)
     verifynum = 0
     for nzf in nzf_list:
-        verifynum += 1
-        nzo.set_action_line(T("Verifying"), "%02d/%02d" % (verifynum, verifytotal))
-        calculated_crc32[nzf.filename] = crc_calculate(os.path.join(nzo.download_path, nzf.filename))
+        if nzf.crc32 is not None:
+            verifynum += 1
+            nzo.set_action_line(T("Verifying"), "%02d/%02d" % (verifynum, verifytotal))
+            calculated_crc32[nzf.filename] = b"%08x" % (nzf.crc32 & 0xFFFFFFFF)
 
     sfv_parse_results = {}
     nzo.set_action_line(T("Trying SFV verification"), "...")
@@ -2176,7 +2164,7 @@ def sfv_check(sfvs: List[str], nzo: NzbObject) -> bool:
             # Do a simple filename based check
             if file == nzf.filename:
                 found = True
-                if nzf.filename in calculated_crc32 and calculated_crc32[nzf.filename] == sfv_parse_results[file]:
+                if calculated_crc32.get(nzf.filename, "") == sfv_parse_results[file]:
                     logging.debug("SFV-check of file %s OK", file)
                     result &= True
                 elif file_to_ignore:
@@ -2189,7 +2177,7 @@ def sfv_check(sfvs: List[str], nzo: NzbObject) -> bool:
                 break
 
             # Now lets do obfuscation check
-            if nzf.filename in calculated_crc32 and calculated_crc32[nzf.filename] == sfv_parse_results[file]:
+            if calculated_crc32.get(nzf.filename, "") == sfv_parse_results[file]:
                 try:
                     logging.debug("SFV-check will rename %s to %s", nzf.filename, file)
                     renamer(os.path.join(nzo.download_path, nzf.filename), os.path.join(nzo.download_path, file))
@@ -2205,7 +2193,7 @@ def sfv_check(sfvs: List[str], nzo: NzbObject) -> bool:
         if not found:
             if file_to_ignore:
                 # We don't care about these files
-                logging.debug("SVF-check ignoring missing file %s", file)
+                logging.debug("SFV-check ignoring missing file %s", file)
                 continue
 
             logging.info("Cannot SFV-check missing file %s!", file)
@@ -2234,18 +2222,6 @@ def parse_sfv(sfv_filename):
     return results
 
 
-def crc_calculate(path):
-    """Calculate crc32 of the given file"""
-    crc = 0
-    with open(path, "rb") as fp:
-        while 1:
-            data = fp.read(4096)
-            if not data:
-                break
-            crc = zlib.crc32(data, crc)
-    return b"%08x" % (crc & 0xFFFFFFFF)
-
-
 def add_time_left(perc: float, start_time: Optional[float] = None, time_used: Optional[float] = None) -> str:
     """Calculate time left based on current progress, if it is taking more than 10 seconds"""
     if not time_used:
@@ -2255,18 +2231,39 @@ def add_time_left(perc: float, start_time: Optional[float] = None, time_used: Op
     return ""
 
 
-def analyse_show(name: str) -> Tuple[str, str, str, str, bool]:
-    """Do a quick SeasonSort check and return basic facts"""
-    job = SeriesSorter(None, name, None, None, force=True)
-    if job.matched:
-        job.get_values()
-    return (
-        job.info.get("title", ""),
-        job.info.get("season_num", ""),
-        job.info.get("episode_num", ""),
-        job.info.get("ep_name", ""),
-        job.is_proper(),
+def analyse_show(name: str) -> Dict[str, str]:
+    """Use the Sorter to collect some basic info on series"""
+    job = Sorter(
+        None,
+        name,
+        None,
+        None,
+        force=True,
+        sorter_config={
+            "name": "newsunpack__analyse_show",
+            "order": 0,
+            "min_size": -1,
+            "multipart_label": "",
+            "sort_string": "",
+            "sort_cats": [],  # Categories and types are ignored when using the force
+            "sort_type": [],
+            "is_active": 1,
+        },
     )
+    job.get_values()
+    return {
+        "title": job.info.get("title", ""),
+        "season": job.info.get("season_num", ""),
+        "episode": job.info.get("episode_num", ""),
+        "episode_name": job.info.get("ep_name", ""),
+        "is_proper": str(job.is_proper()),
+        "resolution": job.info.get("resolution", ""),
+        "decade": job.info.get("decade", ""),
+        "year": job.info.get("year", ""),
+        "month": job.info.get("month", ""),
+        "day": job.info.get("day", ""),
+        "job_type": job.type,
+    }
 
 
 def pre_queue(nzo: NzbObject, pp, cat):
@@ -2294,7 +2291,7 @@ def pre_queue(nzo: NzbObject, pp, cat):
             str(nzo.bytes),
             " ".join(nzo.groups),
         ]
-        command.extend(analyse_show(nzo.final_name_with_password)[:4])
+        command.extend(list(analyse_show(nzo.final_name_with_password).values()))
         command = [fix(arg) for arg in command]
 
         # Fields not in the NZO directly
@@ -2304,6 +2301,13 @@ def pre_queue(nzo: NzbObject, pp, cat):
             "show_season": command[9],
             "show_episode": command[10],
             "show_episode_name": command[11],
+            "proper": command[12],
+            "resolution": command[13],
+            "decade": command[14],
+            "year": command[15],
+            "month": command[16],
+            "day": command[17],
+            "type": command[18],
         }
 
         try:
@@ -2312,7 +2316,7 @@ def pre_queue(nzo: NzbObject, pp, cat):
             logging.debug("Failed script %s, Traceback: ", script_path, exc_info=True)
             return values
 
-        output = platform_btou(p.stdout.read())
+        output = p.stdout.read()
         ret = p.wait()
         logging.info("Pre-queue script returned %s and output=\n%s", ret, output)
         if ret == 0:
@@ -2380,7 +2384,7 @@ class SevenZip:
         """Read named file from 7Zip and return data"""
         command = [SEVENZIP_COMMAND, "e", "-p", "-y", "-so", self.path, name]
         # Ignore diagnostic output, otherwise it will be appended to content
-        with build_and_run_command(command, stderr=subprocess.DEVNULL) as p:
+        with build_and_run_command(command, text_mode=False, stderr=subprocess.DEVNULL) as p:
             data = io.BytesIO(p.stdout.read())
             p.wait()
         return data
