@@ -28,6 +28,7 @@ import xml.etree.ElementTree
 import datetime
 import zipfile
 import tempfile
+
 import cherrypy._cpreqbody
 from typing import Optional, Dict, Any, Union, List, Tuple
 
@@ -43,7 +44,7 @@ from sabnzbd.filesystem import (
     remove_data,
 )
 from sabnzbd.misc import name_to_cat
-from sabnzbd.constants import DEFAULT_PRIORITY, VALID_ARCHIVES
+from sabnzbd.constants import DEFAULT_PRIORITY, VALID_ARCHIVES, AddNzbFileResult
 from sabnzbd.utils import rarfile
 
 
@@ -158,11 +159,9 @@ def process_nzb_archive_file(
     url: Optional[str] = None,
     password: Optional[str] = None,
     nzo_id: Optional[str] = None,
-) -> Tuple[int, List[str]]:
+) -> Tuple[AddNzbFileResult, List[str]]:
     """Analyse archive and create job(s).
     Accepts archive files with ONLY nzb/nfo/folder files in it.
-    returns (status, nzo_ids)
-        status: -2==Error/retry, -1==Error, 0==OK, 1==No files found
     """
     nzo_ids = []
     if catdir is None:
@@ -178,21 +177,21 @@ def process_nzb_archive_file(
             zf = sabnzbd.newsunpack.SevenZip(path)
         else:
             logging.info("File %s is not a supported archive!", filename)
-            return -1, []
+            return AddNzbFileResult.ERROR, []
     except:
         logging.info(T("Cannot read %s"), path, exc_info=True)
-        return -2, []
+        return AddNzbFileResult.RETRY, []
 
-    status = 1
+    status: AddNzbFileResult = AddNzbFileResult.NO_FILES_FOUND
     names = zf.namelist()
     nzbcount = 0
     for name in names:
         name = name.lower()
         if name.endswith(".nzb"):
-            status = 0
+            status = AddNzbFileResult.OK
             nzbcount += 1
 
-    if status == 0:
+    if status == AddNzbFileResult.OK:
         if nzbcount != 1:
             nzbname = None
         for name in names:
@@ -202,7 +201,7 @@ def process_nzb_archive_file(
                 except OSError:
                     logging.error(T("Cannot read %s"), name, exc_info=True)
                     zf.close()
-                    return -1, []
+                    return AddNzbFileResult.ERROR, []
                 name = get_filename(name)
                 if datap:
                     nzo = None
@@ -254,7 +253,7 @@ def process_nzb_archive_file(
 
     # If all were rejected/empty/etc, update status
     if not nzo_ids:
-        status = 1
+        status = AddNzbFileResult.NO_FILES_FOUND
 
     return status, nzo_ids
 
@@ -275,11 +274,9 @@ def process_single_nzb(
     url: Optional[str] = None,
     password: Optional[str] = None,
     nzo_id: Optional[str] = None,
-) -> Tuple[int, List[str]]:
+) -> Tuple[AddNzbFileResult, List[str]]:
     """Analyze file and create a job from it
     Supports NZB, NZB.BZ2, NZB.GZ and GZ.NZB-in-disguise
-    returns (status, nzo_ids)
-        status: -2==Error/retry, -1==Error, 0==OK
     """
     if catdir is None:
         catdir = cat
@@ -302,7 +299,7 @@ def process_single_nzb(
     except OSError:
         logging.warning(T("Cannot read %s"), clip_path(path))
         logging.info("Traceback: ", exc_info=True)
-        return -2, []
+        return AddNzbFileResult.RETRY, []
 
     if filename:
         filename, cat = name_to_cat(filename, catdir)
@@ -312,7 +309,7 @@ def process_single_nzb(
             nzbname = get_filename(filename)
 
     # Parse the data
-    result = 0
+    result: AddNzbFileResult = AddNzbFileResult.OK
     nzo = None
     nzo_ids = []
     try:
@@ -332,17 +329,19 @@ def process_single_nzb(
         )
         if not nzo.password:
             nzo.password = password
-    except (sabnzbd.nzbstuff.NzbEmpty, sabnzbd.nzbstuff.NzbRejected):
-        # Empty or fully rejected
-        result = -1
-        pass
+    except sabnzbd.nzbstuff.NzbEmpty:
+        # Malformed or might not be an NZB file
+        result = AddNzbFileResult.NO_FILES_FOUND
+    except sabnzbd.nzbstuff.NzbRejected:
+        # Rejected as duplicate or by pre-queue script
+        result = AddNzbFileResult.ERROR
     except sabnzbd.nzbstuff.NzbRejectedToHistory as err:
         # Duplicate or unwanted extension that was failed to history
         nzo_ids.append(err.nzo_id)
     except:
         # Something else is wrong, show error
         logging.error(T("Error while adding %s, removing"), filename, exc_info=True)
-        result = -1
+        result = AddNzbFileResult.ERROR
     finally:
         nzb_fp.close()
 
@@ -350,7 +349,7 @@ def process_single_nzb(
         nzo_ids.append(sabnzbd.NzbQueue.add(nzo, quiet=bool(reuse)))
 
     try:
-        if not keep:
+        if not keep and result in {AddNzbFileResult.ERROR, AddNzbFileResult.OK}:
             remove_file(path)
     except OSError:
         # Job was still added to the queue, so throw error but don't report failed add
@@ -449,7 +448,6 @@ def nzbfile_parser(full_nzb_path: str, nzo):
                                     logging.info("Skipping duplicate article (%s)", article_id)
                             elif segment_size <= 0 or segment_size >= 2**23:
                                 # Perform sanity check (not negative, 0 or larger than 8MB) on article size
-                                # We use this value later to allocate memory in cache and sabyenc
                                 logging.info("Skipping article %s due to strange size (%s)", article_id, segment_size)
                                 nzo.increase_bad_articles_counter("bad_articles")
                                 bad_articles = True
@@ -469,14 +467,12 @@ def nzbfile_parser(full_nzb_path: str, nzo):
                 # Check if we already have this exact NZF (see custom eq-checks)
                 if nzf in nzo.files:
                     logging.info("File %s occurred twice in NZB, skipping", nzf.filename)
+                    remove_data(nzf.nzf_id, nzo.admin_path)
                     continue
 
                 # Add valid NZF's
                 if file_name and nzf.valid and nzf.nzf_id:
-                    logging.info("File %s added to queue", nzf.filename)
-                    nzo.files.append(nzf)
-                    nzo.files_table[nzf.nzf_id] = nzf
-                    nzo.bytes += nzf.bytes
+                    nzo.add_nzf(nzf)
                     valid_files += 1
                     avg_age_sum += file_timestamp
                 else:
