@@ -1,5 +1,5 @@
 #!/usr/bin/python3 -OO
-# Copyright 2007-2022 The SABnzbd-Team <team@sabnzbd.org>
+# Copyright 2007-2023 The SABnzbd-Team <team@sabnzbd.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -31,7 +31,7 @@ with open("sabnzbd/version.py") as version_file:
 HEADER = (
     r"""#
 # SABnzbd Translation Template file __TYPE__
-# Copyright 2007-2022 The SABnzbd-Team
+# Copyright 2007-2023 The SABnzbd-Team
 #   team@sabnzbd.org
 #
 msgid ""
@@ -140,7 +140,7 @@ if not os.path.exists(PO_DIR):
 
 # Determine location of PyGetText tool
 path, py = os.path.split(sys.argv[0])
-PYGETTEXT = os.path.abspath(os.path.normpath(os.path.join(path, "pygettext.py")))
+PYGETTEXT = os.path.normpath(os.path.join(path, "pygettext.py"))
 cmd = "%s %s %s %s" % (sys.executable, PYGETTEXT, PARMS, FILES)
 os.system(cmd)
 print("Finished creating POT file")
